@@ -1324,6 +1324,7 @@ static void
 draw_keyboard (GkbdKeyboardDrawing * drawing)
 {
 	GtkStateType state = gtk_widget_get_state (GTK_WIDGET (drawing));
+        GtkStyle *style = gtk_widget_get_style (GTK_WIDGET (drawing));
 	GtkAllocation allocation;
 
 	if (!drawing->xkb)
@@ -1335,14 +1336,12 @@ draw_keyboard (GkbdKeyboardDrawing * drawing)
 	    gdk_pixmap_new (gtk_widget_get_window (GTK_WIDGET (drawing)),
 			    allocation.width, allocation.height, -1);
 
-	/* blank background */
-	gdk_draw_rectangle (drawing->pixmap,
-			    gtk_widget_get_style (GTK_WIDGET
-						  (drawing))->base_gc
-			    [state], TRUE, 0, 0, allocation.width,
-			    allocation.height);
-
 	if (create_cairo (drawing)) {
+	        /* blank background */
+                gdk_cairo_set_source_color (drawing->renderContext->cr,
+                                            &style->base[state]);
+                cairo_paint (drawing->renderContext->cr);
+
 		draw_keyboard_to_context (drawing->renderContext, drawing);
 		destroy_cairo (drawing);
 	}
@@ -1384,8 +1383,8 @@ static gboolean
 expose_event (GtkWidget * widget,
 	      GdkEventExpose * event, GkbdKeyboardDrawing * drawing)
 {
-	GtkStateType state = gtk_widget_get_state (GTK_WIDGET (drawing));
 	GtkAllocation allocation;
+        cairo_t *cr;
 
 	if (!drawing->xkb)
 		return FALSE;
@@ -1393,12 +1392,14 @@ expose_event (GtkWidget * widget,
 	if (drawing->pixmap == NULL)
 		return FALSE;
 
-	gdk_draw_drawable (gtk_widget_get_window (widget),
-			   gtk_widget_get_style (widget)->fg_gc[state],
-			   drawing->pixmap,
-			   event->area.x, event->area.y,
-			   event->area.x, event->area.y,
-			   event->area.width, event->area.height);
+        cr = gdk_cairo_create (event->window);
+        gdk_cairo_region (cr, event->region);
+        cairo_clip (cr);
+
+        gdk_cairo_set_source_pixmap (cr, drawing->pixmap, 0, 0);
+        cairo_paint (cr);
+
+        cairo_destroy (cr);
 
 	if (gtk_widget_has_focus (widget)) {
 		gtk_widget_get_allocation (widget, &allocation);
